"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
const lodashDebounce = require("lodash.debounce");
const highlighter_1 = require("./highlighter");
const theme_1 = require("./theme");
__export(require("./theme"));
__export(require("./language"));
__export(require("./highlighter"));
__export(require("./highlight"));
function monacoPositionToParserPoint(position) {
    return { row: position.lineNumber, column: position.column };
}
class MonacoTreeSitter {
    constructor(editor, language) {
        this.editor = editor;
        this.language = language;
        this.monacoDecorationKeys = [];
        this.buildHighlightDebounced = lodashDebounce(this.buildHighlight.bind(this), 20);
        this.tree = language.parser.parse(editor.getValue());
        editor.getModel().onDidChangeContent(this.onEditorContentChange.bind(this));
        this.buildHighlight();
    }
    onEditorContentChange(e) {
        if (e.changes.length == 0)
            return;
        for (const change of e.changes) {
            const startIndex = change.rangeOffset;
            const oldEndIndex = change.rangeOffset + change.rangeLength;
            const newEndIndex = change.rangeOffset + change.text.length;
            const startPosition = monacoPositionToParserPoint(this.editor.getModel().getPositionAt(startIndex));
            const oldEndPosition = monacoPositionToParserPoint(this.editor.getModel().getPositionAt(oldEndIndex));
            const newEndPosition = monacoPositionToParserPoint(this.editor.getModel().getPositionAt(newEndIndex));
            this.tree.edit({ startIndex, oldEndIndex, newEndIndex, startPosition, oldEndPosition, newEndPosition });
        }
        this.tree = this.language.parser.parse(this.editor.getValue(), this.tree);
        this.buildHighlightDebounced();
    }
    buildHighlight() {
        const decorations = highlighter_1.buildDecorations(this.tree, this.language);
        const monacoDecorations = [];
        for (const [term, ranges] of Object.entries(decorations)) {
            const options = {
                inlineClassName: theme_1.Theme.getClassNameOfTerm(term)
            };
            for (const range of ranges) {
                monacoDecorations.push({ range, options });
            }
        }
        this.monacoDecorationKeys = this.editor.deltaDecorations(this.monacoDecorationKeys, monacoDecorations);
    }
    changeLanguage(language) {
        this.language = language;
        this.tree = language.parser.parse(this.editor.getValue());
        this.buildHighlight();
    }
    refresh() {
        this.buildHighlight();
    }
}
exports.MonacoTreeSitter = MonacoTreeSitter;
//# sourceMappingURL=index.js.map